@@ -74,8 +74,8 @@ class TestStubbing < Minitest::Test
 
   def test_stubbed_methods_can_take_blocks
     obj = Object.new
-    flexmock(obj).should_receive(:with_block).once.with(Proc).
-      and_return { |block| block.call }
+    flexmock(obj).should_receive(:with_block).once.with_block.
+      and_return { |&block| block.call }
     assert_equal :block, obj.with_block { :block }
   end
 
@@ -92,6 +92,18 @@ class TestStubbing < Minitest::Test
     assert_equal :woof, dog.bark
   end
 
+  def test_stubbed_methods_handle_singleton_methods_added_after_the_mock_was_created
+    dog = Dog.new
+    m = Module.new do
+      def bark
+        :baaaaaark
+      end
+    end
+    flexmock(dog).should_receive(:bark).pass_thru.once
+    dog.extend m
+    assert_equal :baaaaaark, dog.bark
+  end
+
   def test_invoke_original_allows_to_call_the_original_directly
     dog = Dog.new
     flexmock(dog).should_receive(:bark).never
@@ -622,7 +634,7 @@ class TestStubbing < Minitest::Test
     exception = assert_raises(NameError) do
         obj.mocked_method
     end
-    assert_equal "undefined method `does_not_exist' for #{obj}", exception.message
+    assert_match /undefined method `does_not_exist' for/, exception.message
   end
 
   def test_it_checks_whether_mocks_are_forbidden_before_forwarding_the_call
@@ -740,6 +752,20 @@ class TestStubbing < Minitest::Test
     assert_equal 1, obj.value
   end
 
+  def test_pass_thru_forwards_a_given_block
+    obj = Class.new do
+      attr_reader :block
+      def mocked_method(&block)
+        @block = block
+      end
+    end.new
+    flexmock(obj).should_receive(:mocked_method).with_block.pass_thru
+
+    block = obj.mocked_method { 42 }
+    assert_kind_of Proc, block
+    assert_equal block, obj.block
+  end
+
   def test_should_expect
     flexmock(obj = Dog.new).should_expect do |e|
       e.bark
@@ -757,5 +783,50 @@ class TestStubbing < Minitest::Test
     FlexMock.partials_are_based = false
     FlexMock.partials_verify_signatures = false
   end
+
+  def test_can_call_the_original_method_from_and_return
+    obj = Dog.new
+    flexmock(obj)
+    called = false
+    obj.should_receive(:bark).once.and_return do
+      called = true
+      obj.invoke_original(:bark)
+    end
+    assert_equal :woof, obj.bark
+    assert called
+  end
+
+  class PartialsNewSignature
+    def initialize(arg, with_kw: nil)
+    end
+  end
+
+  def test_it_lets_new_be_called_if_the_signature_matches_initialize
+    FlexMock.partials_verify_signatures = true
+
+    flexmock(PartialsNewSignature)
+      .should_receive(:new).with(10, with_kw: 20)
+
+    PartialsNewSignature.new(10, with_kw: 20)
+  ensure
+    FlexMock.partials_verify_signatures = true
+  end
+
+  def test_it_fails_the_signature_check_if_a_call_to_new_is_compatible_with_the_signature_of_initialize
+    FlexMock.partials_verify_signatures = true
+
+    flexmock(PartialsNewSignature)
+      .should_receive(:new).with(10, something: 20)
+
+    e = assert_raises(FlexMock::CheckFailedError) do
+      PartialsNewSignature.new(10, something: 20)
+    end
+    assert_match(
+      /given.unexpected.keyword.argument.something/,
+      e.message
+    )
+  ensure
+    FlexMock.partials_verify_signatures = true
+  end
 end
 
