@@ -88,9 +88,9 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
-  def test_block_example_from_readme
+  def test_with_block_example_from_readme
     FlexMock.use do |m|
-      m.should_receive(:foo).with(Integer,Proc).and_return(:got_block)
+      m.should_receive(:foo).with(Integer).with_block.and_return(:got_block)
       m.should_receive(:foo).with(Integer).and_return(:no_block)
 
       assert_equal :no_block, m.foo(1)
@@ -98,6 +98,25 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
+  def test_with_no_block_example_from_readme
+    FlexMock.use do |m|
+      m.should_receive(:foo).with(Integer).with_no_block.and_return(:no_block)
+      m.should_receive(:foo).with(Integer).and_return(:got_block)
+
+      assert_equal :no_block, m.foo(1)
+      assert_equal :got_block, m.foo(1) { }
+    end
+  end
+
+  def test_with_optional_block
+    FlexMock.use do |m|
+      m.should_receive(:foo).with(Integer).with_optional_block.twice
+
+      m.foo(1)
+      m.foo(1) {}
+    end
+  end
+
   def test_return_with_and_without_block_interleaved
     FlexMock.use do |m|
       m.should_receive(:hi).and_return(:a).and_return { :b }.and_return(:c)
@@ -421,10 +440,31 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
+  def test_with_kw_matching
+    FlexMock.use('greeter') do |m|
+      m.should_receive(:hi).with(1, a: 2).once
+      m.hi(1, a: 2)
+    end
+
+    FlexMock.use('greeter') do |m|
+      m.should_receive(:hi).with_kw_args(FlexMock.hsh(:a => 1, :b => 2)).once
+      m.hi(:a => 1, :b => 2, :c => 3)
+    end
+  end
+
+  def test_with_kw_not_matching
+    FlexMock.use('greeter') do |m|
+      m.should_receive(:hi).with(1, a: 2)
+      assert_mock_failure(check_failed_error, :deep => true, :line => __LINE__+1) {
+        m.hi(1, a: 2, b: 3)
+      }
+    end
+  end
+
   def test_with_hash_matching
     FlexMock.use('greeter') do |m|
       m.should_receive(:hi).with(FlexMock.hsh(:a => 1, :b => 2)).once
-      m.hi(:a => 1, :b => 2, :c => 3)
+      m.hi({:a => 1, :b => 2, :c => 3}, **{})
     end
   end
 
@@ -432,7 +472,7 @@ class TestFlexMockShoulds < Minitest::Test
     FlexMock.use('greeter') do |m|
       m.should_receive(:hi).with(FlexMock.hsh(:a => 1, :b => 2))
       assert_mock_failure(check_failed_error, :deep => true, :line => __LINE__+1) {
-        m.hi(:a => 1, :b => 4, :c => 3)
+        m.hi({:a => 1, :b => 4, :c => 3}, **{})
       }
     end
   end
@@ -446,28 +486,6 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
-  def test_with_optional_proc
-    FlexMock.use('greeter') do |m|
-      m.should_receive(:hi).with(optional_proc).once
-      m.hi { }
-    end
-  end
-
-  def test_with_optional_proc_and_missing_proc
-    FlexMock.use('greeter') do |m|
-      m.should_receive(:hi).with(optional_proc).once
-      m.hi
-    end
-  end
-
-  def test_with_optional_proc_distinquishes_between_nil_and_missing
-    FlexMock.use('greeter') do |m|
-      m.should_receive(:hi).with(optional_proc).never
-      m.should_receive(:hi).with(nil).once
-      m.hi(nil)
-    end
-  end
-
   def test_with_arbitrary_arg_matching
     FlexMock.use('greeter') do |m|
       m.should_receive(:hi).with(FlexMock.on { |arg| arg % 2 == 0 rescue nil }).twice
@@ -531,9 +549,9 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
-  def test_block_arg_given_to_no_args
+  def test_block_arg_given_to_no_block
     FlexMock.use do |m|
-      m.should_receive(:hi).with_no_args.returns(20)
+      m.should_receive(:hi).with_no_block.returns(20)
       assert_mock_failure(check_failed_error, :message =>NO_MATCH_ERROR_MESSAGE, :deep => true, :line => __LINE__+1) {
         m.hi { 1 }
       }
@@ -543,8 +561,9 @@ class TestFlexMockShoulds < Minitest::Test
   def test_block_arg_given_to_matching_proc
     FlexMock.use do |m|
       arg = nil
-      m.should_receive(:hi).with(Proc).once.
-        and_return { |block| arg = block; block.call }
+      m.should_receive(:hi)
+       .with_block.once
+       .and_return { |&block| arg = block; block.call }
       result = m.hi { 1 }
       assert_equal 1, arg.call
       assert_equal 1, result
@@ -1208,11 +1227,14 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
-  def test_a_proc_argument_last_can_be_interpreted_as_positional_argument
+  def test_a_proc_argument_last_is_not_interpreted_as_positional_argument
     FlexMock.use do |mock|
       mock.should_receive(:m).with_signature(required_arguments: 2)
-      mock.m(1) { }
       mock.m(1, 2) { }
+
+      assert_raises(FlexMock::CheckFailedError) do
+        mock.m(1) { }
+      end
     end
   end
 
@@ -1231,14 +1253,6 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
-  def test_it_interprets_a_hash_as_last_positional_argument_if_no_keyword_arguments_are_expected
-    FlexMock.use do |mock|
-      mock = flexmock
-      mock.should_receive(:m).with_signature(required_arguments: 2, optional_arguments: 2)
-      mock.m(1, 2, 3, test: 4)
-    end
-  end
-
   def test_with_signature_splat_validates_required_arguments
     FlexMock.use do |mock|
       mock = flexmock
@@ -1346,14 +1360,6 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
-  def test_signature_validator_understands_that_a_hash_can_be_used_for_both_keywords_and_positional_arguments
-      FlexMock.use do |mock|
-        mock.should_receive(:m).
-          with_signature(optional_keyword_arguments: [:b], required_arguments: 1)
-        mock.m(Hash.new)
-      end
-  end
-
   def test_signature_validator_understands_that_a_proc_last_can_both_be_a_positional_parameter_and_a_block
       FlexMock.use do |mock|
         mock.should_receive(:m).with_signature(required_arguments: 1)
@@ -1377,7 +1383,12 @@ class TestFlexMockShoulds < Minitest::Test
       FlexMock.use do |mock|
         mock.should_receive(:m).with_signature(required_arguments: 1, required_keyword_arguments: [:b])
         mock.m(10, b: 10) { }
-        assert_mock_failure(check_failed_error, message: /in mock 'unknown': m\(\*args\) expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
+        signature = if RUBY_VERSION < "3"
+                      'm\(\*args\)'
+                    else
+                      'm\(\*args, \*\*kwargs\)'
+                    end
+        assert_mock_failure(check_failed_error, message: /in mock 'unknown': #{signature} expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
           mock.m(b: 10) { }
         end
       end
@@ -1394,10 +1405,14 @@ class TestFlexMockShoulds < Minitest::Test
       FlexMock.use do |mock|
         mock.should_receive(:m).
           with_signature(required_keyword_arguments: [:b], required_arguments: 1)
-        assert_mock_failure(check_failed_error, message: /in mock 'unknown': m\(\*args\) expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
-          mock.m(Hash.new)
+        signature = 'm\(\*args, \*\*kwargs\)'
+        if RUBY_VERSION < "3"
+          signature = 'm\(\*args\)'
+          assert_mock_failure(check_failed_error, message: /in mock 'unknown': #{signature} expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
+            mock.m({ :foo => "bar" })
+          end
         end
-        assert_mock_failure(check_failed_error, message: /in mock 'unknown': m\(\*args\) expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
+        assert_mock_failure(check_failed_error, message: /in mock 'unknown': #{signature} expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
           mock.m(b: 10)
         end
       end
@@ -1484,4 +1499,3 @@ end
 if Gem::Version.new(RUBY_VERSION) >= Gem::Version.new('2.1')
     require_relative 'should_receive_ruby21plus'
 end
-
