@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 #---
-# Copyright 2003-2011 by Jim Weirich (jim@weirichhouse.org).
+# Copyright 2003-2013 by Jim Weirich (jim.weirich@gmail.com).
 # All rights reserved.
 
 # Permission is granted for use, copying, modification, distribution,
@@ -9,14 +9,20 @@
 # above copyright notice is included.
 #+++
 
-require "test/test_setup"
+require "test_helper"
 
 require "flexmock/base"
 require "flexmock/test_unit"
 
 class TestFlexmockTestUnit < Test::Unit::TestCase
   def teardown
-    super
+    failed = false
+    begin
+      super
+    rescue Exception => ex
+      failed = true
+    end
+    assert_equal @should_fail, failed, "Expected failed to be #{@should_fail}"
   end
 
   # This test should pass.
@@ -24,11 +30,13 @@ class TestFlexmockTestUnit < Test::Unit::TestCase
     m = flexmock("mock")
     m.should_receive(:hi).once
     m.hi
+    @should_fail = false
   end
 
   # This test should fail during teardown.
   def test_should_fail__mocks_are_auto_verified
     m = flexmock("mock")
     m.should_receive(:hi).once
+    @should_fail = true
   end
 end
